// this file is part of the Quirk browser
// copyright 2021 by zamfofex
// license: GPLv3 or later

import {Generator, fetch} from "@jspm/generator"
import fs from "fs/promises"

let packages =
[
	"jsdom@17.0.0",
	
	"css-typed-om@0.4.0",
	"@stardazed/streams-fetch-adapter@3.0.0",
	"whatwg-fetch@3.6.2",
	"whatwg-url@9.1.0",
	{target: "web-streams-polyfill@3.1.1", subpath: './ponyfill'},
	
	"assert@2.0.0",
	"events@3.3.0",
	"process@0.11.10",
	"punycode@2.1.1",
	"stream-http@3.2.0",
	"string_decoder@1.3.0",
	"tty-browserify@0.0.1",
	"url@0.11.0",
	"util@0.12.4",
	{alias: "zlib", target: "browserify-zlib@0.2.0"},
	{alias:"crypto", target: "crypto-browserify@3.12.0"},
	{alias: "path", target: "path-browserify@1.0.1"},
	{alias: "querystring", target: "querystring-es3@1.0.0-0"},
	{alias: "stream", target: "stream-browserify@3.0.0"},
]

let generator = new Generator({env: ["production", "browser", "module"]})

let {staticDeps, dynamicDeps} = await generator.install(packages)
let set = new Set([...staticDeps, ...dynamicDeps])

let map = generator.getMap()

let quirk = await fs.open("js/modules.bin", "w")

let fd = await fs.open("dom/modules.bin", "w")
let delim = "\0"

let out = async (prefix, name, url) =>
{
	let response = await fetch(url)
	let js = await response.text()
	
	set.delete(url)
	url = new URL(url).pathname.slice(1)
	
	if (url.startsWith("npm:jsdom@17.0.0/"))
		js = js.replace(/={implementation:EventTargetImpl}/, ".implementation=EventTargetImpl")
	
	await fs.writeFile(fd, prefix)
	await fs.writeFile(fd, delim)
	
	await fs.writeFile(fd, name)
	await fs.writeFile(fd, delim)
	
	await fs.writeFile(fd, url)
	await fs.writeFile(fd, delim)
	
	await fs.writeFile(fd, js)
	await fs.writeFile(fd, delim)
}

for (let name of await fs.readdir("js"))
{
	if (!name.endsWith(".js"))
		continue
	
	await fs.writeFile(quirk, "quirk/")
	await fs.writeFile(quirk, delim)
	
	await fs.writeFile(quirk, `quirk/${name}`)
	await fs.writeFile(quirk, delim)
	
	await fs.writeFile(quirk, `quirk/${name}`)
	await fs.writeFile(quirk, delim)
	
	await fs.writeFile(quirk, await fs.readFile(`js/${name}`))
	await fs.writeFile(quirk, delim)
}

for (let name of await fs.readdir("js/shims"))
{
	if (!name.endsWith(".js"))
		continue
	
	await fs.writeFile(quirk, delim)
	
	await fs.writeFile(quirk, name.slice(0, -3))
	await fs.writeFile(quirk, delim)
	
	await fs.writeFile(quirk, `quirk/shims/${name}`)
	await fs.writeFile(quirk, delim)
	
	await fs.writeFile(quirk, await fs.readFile(`js/shims/${name}`))
	await fs.writeFile(quirk, delim)
}

for (let [name, url] of Object.entries(map.imports))
	await out("", name, url)
for (let [prefix, scoped] of Object.entries(map.scopes))
for (let [name, url] of Object.entries(scoped))
	await out(new URL(prefix).pathname.slice(1), name, url)
for (let url of [...set])
	await out("", url, url)

await fd.close()
await quirk.close()
