// this file is part of the Quirk browser
// copyright 2021 by zamfofex
// license: GPLv3 or later

export {Headers, Request} from "whatwg-fetch"
import {Response as XResponse} from "whatwg-fetch"
import poll from "./poll.js"
import {createAdaptedResponse} from "@stardazed/streams-fetch-adapter"
import {ReadableStream} from "web-streams-polyfill/ponyfill"
import {cURL} from "builtin"

let optionNames = ["method", "headers", "body", "mode", "credentials", "cache", "redirect"]

export let Response = createAdaptedResponse(XResponse, undefined, ReadableStream)

export let fetch = async (request, init) =>
{
	let options = {}
	for (let name of optionNames)
		if (name in init)
			options[name] = init[name]
	
	if (!("credentials" in options))
		options.credentials = "omit"
	
	request = new Request(request, options)
	
	let url = new URL(request.url)
	
	let resolve
	let promise = new Promise(f => resolve = f)
	
	let status
	let statusText
	let headers
	
	let parseHeader = line =>
	{
		line = line.replace(/[\r\n\t ]+/g, " ")
		line = line.trim()
		
		if (line === "")
		{
			if (status < 300 || status > 399) resolve(new Response(stream, {status, statusText, headers}))
			return
		}
		
		let match = line.match(/^HTTP\/(?:[0-9\.]*) ([0-9]{3})(?: ([^]+))?$/)
		if (match)
		{
			headers = []
			status = Number(match[1])
			statusText = match[2] ?? ""
			return
		}
		
		headers.push(line.split(/: */))
	}
	
	let requestHeaders = [["host", url.host], ...request.headers].map(([name, value]) => `${name.replace(/(^|-)[a-z]/g, ch => ch.toUpperCase())}: ${value}`)
	
	let stream = new ReadableStream({
		start: controller => cURL(request.method, url, requestHeaders, async data => controller.enqueue(data), parseHeader, () => controller.close()),
		pull: () => poll(),
	})
	
	return promise
}
