// this file is part of the Quirk browser
// copyright 2021 by zamfofex
// license: GPLv3 or later

#include <stdlib.h>

#include "layout.h"
#include "flow.h"

#include "../util/syntax.h"

static void qr_tree_actual_intrinsic_sizes(QrBox box);
static void qr_tree_formal_sizes(QrBox box, QrNumber horizontal_percent, QrNumber vertical_percent);
static void qr_tree_layout(QrBox box);

static void qr_box_actual_intrinsic_sizes(QrBox box, QrAxis axis);
static void qr_box_actual_given_sizes(QrBox box, QrAxis axis, QrNumber percent, QrNumber padding_percent);

void qr_layout(box)
	QrBox box;
{
	qr_tree_actual_intrinsic_sizes(box);
	
	box->sizes[qr_horizontal].formal = box->sizes[qr_horizontal].given;
	box->sizes[qr_vertical].formal = box->sizes[qr_vertical].given;
	
	qr_tree_formal_sizes(box, box->sizes[qr_horizontal].formal, box->sizes[qr_vertical].formal);
	qr_tree_layout(box);
	
	box->offsets[qr_horizontal] = 0;
	box->offsets[qr_vertical] = 0;
}

void qr_tree_actual_intrinsic_sizes(box)
	QrBox box;
{
	for (unsigned int i = 0 ; i < box->count ; i++)
	{
		QrBox child = box->children[i];
		qr_tree_actual_intrinsic_sizes(child);
	}
	qr_box_actual_intrinsic_sizes(box, qr_horizontal);
	qr_box_actual_intrinsic_sizes(box, qr_vertical);
}

void qr_box_actual_intrinsic_sizes(box, axis)
	QrBox box;
	QrAxis axis;
{
	// calculate the “formal intrinsic sizes”
	(*box->layout->formal_intrinsic_sizes)(box, axis);
	
	// calculate the “normalized sizes”
	(*box->normalized_sizes)(box, axis);
	
	QrSizes sizes = box->sizes[axis];
	if (sizes.normalized.min == qr_omega)
		sizes.normalized.min = 0;
	
	// calculate the “formal given size”
	if (sizes.given != qr_omega)
	{
		if (sizes.given > sizes.normalized.max)
			sizes.given = sizes.normalized.max;
		if (sizes.given < sizes.normalized.min)
			sizes.given = sizes.normalized.min;
	}
	
	// calculate the “actual intrinsic sizes”
	if (sizes.given == qr_omega)
		sizes.actual_intrinsic = sizes.formal_intrinsic;
	else
		sizes.actual_intrinsic.min = sizes.given,
		sizes.actual_intrinsic.max = sizes.given;
	
	// commit calculations
	box->sizes[axis] = sizes;
}

void qr_tree_formal_sizes(box, horizontal_percent, vertical_percent)
	QrBox box;
	QrNumber horizontal_percent;
	QrNumber vertical_percent;
{
	QrNumber padding_percent;
	if (box->inline_axis == qr_vertical)
		padding_percent = vertical_percent;
	else
		padding_percent = horizontal_percent;
	
	qr_box_actual_given_sizes(box, qr_horizontal, horizontal_percent, padding_percent);
	qr_box_actual_given_sizes(box, qr_vertical, vertical_percent, padding_percent);
	
	if (box->sizes[qr_horizontal].given != qr_omega)
		box->sizes[qr_horizontal].formal = box->sizes[qr_horizontal].given;
	if (box->sizes[qr_vertical].given != qr_omega)
		box->sizes[qr_vertical].formal = box->sizes[qr_vertical].given;
	
	(*box->layout->formal_sizes)(box);
	
	for (unsigned int i = 0 ; i < box->count ; i++)
	{
		QrBox child = box->children[i];
		qr_tree_formal_sizes(child, box->sizes[qr_horizontal].formal, box->sizes[qr_vertical].formal);
	}
}

void qr_tree_layout(box)
	QrBox box;
{
	for (unsigned int i = 0 ; i < box->count ; i++)
	{
		QrBox child = box->children[i];
		qr_tree_layout(child);
	}
	
	(*box->layout->layout)(box);
}

void qr_box_actual_given_sizes(box, axis, percent, padding_percent)
	QrBox box;
	QrAxis axis;
	QrNumber percent;
	QrNumber padding_percent;
{
	// recalculate the “normalized sizes”
	(*box->normalized_sizes_again)(box, axis, percent, padding_percent);
	
	QrSizes sizes = box->sizes[axis];
	QrNumber given = sizes.given;
	
	// recalculate the “given size”
	if (given > sizes.normalized.max)
		given = sizes.normalized.max;
	if (given < sizes.normalized.min)
		given = sizes.normalized.min;
	
	// commit calculations
	box->sizes[axis].given = given;
}
