// this file is part of the Quirk browser
// copyright 2021 by zamfofex
// license: GPLv3 or later

#include <pthread.h>
#include <stdlib.h>

#include "../util/syntax.h"

#include "threads.h"

typedef struct QrQueueEntry *QrQueueEntry;
struct QrQueueEntry
{
	QrQueueEntry next;
	void *data;
};

struct QrQueue
{
	pthread_mutex_t mutex;
	pthread_cond_t condition;
	QrQueueEntry newest;
	QrQueueEntry oldest;
};

QrQueue qr_queue()
{
	pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
	pthread_cond_t condition = PTHREAD_COND_INITIALIZER;
	
	QrQueue queue = malloc(sizeof *queue);
	qr_assert(queue != NULL);
	
	queue->mutex = mutex;
	queue->condition = condition;
	queue->newest = NULL;
	queue->oldest = NULL;
	
	return queue;
}

void qr_enqueue(queue, data)
	QrQueue queue;
	void *data;
{
	QrQueueEntry entry = malloc(sizeof *entry);
	qr_assert(entry != NULL);
	
	entry->data = data;
	entry->next = NULL;
	
	qr_assert(pthread_mutex_lock(&queue->mutex) == 0);
	
	if (queue->newest != NULL)
	{
		queue->newest->next = entry;
		queue->newest = entry;
	}
	else
	{
		queue->newest = entry;
		qr_assert(pthread_cond_broadcast(&queue->condition) == 0);
	}
	
	qr_assert(pthread_mutex_unlock(&queue->mutex) == 0);
}

void *qr_dequeue(queue)
	QrQueue queue;
{
	qr_assert(pthread_mutex_lock(&queue->mutex) == 0);
	
	void *result = NULL;
	
	if (queue->oldest != NULL)
	{
		QrQueueEntry old = queue->oldest;
		result = old->data;
		queue->oldest = old->next;
		free(old);
		if (queue->oldest == NULL)
			queue->newest = NULL;
	}
	
	qr_assert(pthread_mutex_unlock(&queue->mutex) == 0);
	
	return result;
}

void *qr_peek(queue)
	QrQueue queue;
{
	qr_assert(pthread_mutex_lock(&queue->mutex) == 0);
	
	void *result = NULL;
	if (queue->oldest != NULL)
		result = queue->oldest->data;
	
	qr_assert(pthread_mutex_unlock(&queue->mutex) == 0);
	
	return result;
}

void qr_wait(queue)
	QrQueue queue;
{
	qr_assert(pthread_mutex_lock(&queue->mutex) == 0);
	while (queue->oldest == NULL)
		pthread_cond_wait(&queue->condition, &queue->mutex);
	qr_assert(pthread_mutex_unlock(&queue->mutex) == 0);
}

void qr_queue_free(queue)
	QrQueue queue;
{
	free(queue);
}
